<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

require_once WCSS_PLUGIN_PATH . 'includes/legacy/class-wcss-legacy-shared-cart.php';

class WCSS_Shared_Cart extends WCSS_Legacy_Shared_Cart {

	/**
	 * Contains the shared cart table name.
	 *
	 * @since 2.0.0
	 *
	 * @var string Shared cart table name.
	 */
	protected $table = 'wcss_shared_cart';

	/**
	 * Contains the shared cart data.
	 *
	 * @since 2.0.0
	 *
	 * @var array Shared cart data.
	 */
	protected $data = array(
		'id'                  => 0,
		'cart_key'            => '',
		'cart_value'          => '',
		'cart_shared_by'      => 0,
		'cart_retrieved'      => 0,
		'cart_retrieved_time' => null,
		'cart_created'        => '',
	);

	/**
	 * Class constructor.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Optional. Shared cart key.
	 */
	public function __construct( $cart_key = '' ) {
		parent::__construct();

		if ( $cart_key ) {
			$this->read( $cart_key );
		}
	}

	/**
	 * --------------------------------
	 *  GETTERS METHODS
	 * --------------------------------
	 */

	/**
	 * Get the shared cart ID.
	 *
	 * @since 2.0.0
	 *
	 * @return int Shared cart ID.
	 */
	public function get_id() {
		return absint( $this->data['id'] );
	}

	/**
	 * Get the shared cart key.
	 *
	 * @since 2.0.0
	 *
	 * @return string Shared cart key.
	 */
	public function get_cart_key() {
		return $this->data['cart_key'];
	}

	/**
	 * Get the shared cart value.
	 *
	 * @since 2.0.0
	 *
	 * @return mixed Shared cart value.
	 */
	public function get_cart_value() {
		return maybe_unserialize( $this->data['cart_value'] );
	}

	/**
	 * Get the shared cart by.
	 *
	 * @since 2.0.0
	 *
	 * @return int Shared cart by user ID.
	 */
	public function get_cart_shared_by() {
		return absint( $this->data['cart_shared_by'] );
	}

	/**
	 * Get the shared cart retrieved count.
	 *
	 * @since 2.0.0
	 *
	 * @return int Shared cart retrieved count.
	 */
	public function get_cart_retrieved( $cart_key = '' ) {
		return absint( $this->data['cart_retrieved'] );
	}

	/**
	 * Get the shared cart retrieved time.
	 *
	 * @since 2.0.0
	 *
	 * @return string Shared cart retreived time.
	 */
	public function get_cart_retrieved_time( $cart_key = '' ) {
		return $this->data['cart_retrieved_time'];
	}

	/**
	 * Get the shared cart created time.
	 *
	 * @since 2.0.0
	 *
	 * @return string Shared cart created time.
	 */
	public function get_cart_created() {
		return $this->data['cart_created'];
	}

	/**
	 * --------------------------------
	 *  SETTERS METHODS
	 * --------------------------------
	 */

	/**
	 * Set the shared cart ID.
	 *
	 * @since 2.0.0
	 *
	 * @param int $id Shared cart ID.
	 * @return void
	 */
	public function set_id( $id ) {
		$this->data['id'] = absint( $id );
	}

	/**
	 * Set the shared cart key.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Shared cart key.
	 * @return void
	 */
	public function set_cart_key( $cart_key ) {
		$this->data['cart_key'] = $cart_key;
	}

	/**
	 * Set the shared cart value.
	 *
	 * @since 2.0.0
	 *
	 * @param mixed $cart_value Shared cart value.
	 * @return void
	 */
	public function set_cart_value( $cart_value ) {
		$this->data['cart_value'] = $cart_value;
	}

	/**
	 * Set the shared cart by.
	 *
	 * @since 2.0.0
	 *
	 * @param int $cart_shared_by User ID of the person who shared the cart.
	 * @return void
	 */
	public function set_cart_shared_by( $cart_shared_by ) {
		$this->data['cart_shared_by'] = absint( $cart_shared_by );
	}

	/**
	 * Set the shared cart retrieved count.
	 *
	 * @since 2.0.0
	 *
	 * @param int $cart_retrieved Count of shared cart retrieved.
	 * @return void
	 */
	public function set_cart_retrieved( $cart_retrieved ) {
		$this->data['cart_retrieved'] = absint( $cart_retrieved );
	}

	/**
	 * Set the shared cart last retrieved time.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_retrieved_time Last shared cart retrieved time.
	 * @return void
	 */
	public function set_cart_retrieved_time( $cart_retrieved_time ) {
		$this->data['cart_retrieved_time'] = $cart_retrieved_time;
	}

	/**
	 * Set the shared cart created time.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_created Shared cart creation time.
	 * @return void
	 */
	public function set_cart_created( $cart_created ) {
		$this->data['cart_created'] = $cart_created;
	}

	/**
	 * --------------------------------
	 *  CONDITIONAL METHODS
	 * --------------------------------
	 */

	/**
	 * Check whether the shared cart is exists or not.
	 *
	 * @since 2.0.0
	 *
	 * @return bool True if shared cart exists, false otherwise.
	 */
	public function is_exists() {
		return ( $this->get_id() > 0 );
	}

	/**
	 * --------------------------------
	 *  OTHER METHODS
	 * --------------------------------
	 */

	/**
	 * Generate 5 characters unique alphanumeric key.
	 *
	 * @since 2.0.0
	 *
	 * @return string 5 characters unique alphanumeric key
	 */
	public function generate_cart_key() {

		$charset = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
		$base    = strlen( $charset );
		$result  = '';

		$now = explode( ' ', microtime() )[1];

		while ( $now >= $base ) {
			$i      = $now % $base;
			$result = $charset[ $i ] . $result;
			$now   /= $base;
		}

		return substr( $result, -5 );
	}

	/**
	 * --------------------------------
	 *  CURD METHODS
	 * --------------------------------
	 */

	/**
	 * Create a new shared cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Shared_Cart|bool Current instance of WCSS_Shared_Cart, false otherwise.
	 */
	public function create() {
		global $wpdb;

		if ( ! $this->get_cart_key() ) {
			$this->set_cart_key( $this->generate_cart_key() );
		}
		if ( ! $this->get_cart_shared_by() ) {
			$this->set_cart_shared_by( get_current_user_id() );
		}
		if ( ! $this->get_cart_created() ) {
			$this->set_cart_created( current_time( 'mysql' ) );
		}

		$create = $wpdb->insert(
			$wpdb->prefix . $this->table,
			array(
				'cart_key'            => $this->get_cart_key(),
				'cart_value'          => maybe_serialize( $this->get_cart_value() ),
				'cart_shared_by'      => $this->get_cart_shared_by(),
				'cart_retrieved'      => $this->get_cart_retrieved(),
				'cart_retrieved_time' => $this->get_cart_retrieved_time(),
				'cart_created'        => $this->get_cart_created(),
			)
		);

		if ( ! $create ) {
			return false;
		}

		return $this;
	}

	/**
	 * Read a shared cart from database.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Shared cart key.
	 * @return void
	 */
	public function read( $cart_key ) {
		global $wpdb;

		$read = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->prefix}{$this->table} WHERE cart_key = %s",
				$cart_key
			)
		);

		if ( ! $read ) {
			return;
		}

		$this->set_id( $read->id );
		$this->set_cart_key( $read->cart_key );
		$this->set_cart_value( $read->cart_value );
		$this->set_cart_shared_by( $read->cart_shared_by );
		$this->set_cart_retrieved( $read->cart_retrieved );
		$this->set_cart_retrieved_time( $read->cart_retrieved_time );
		$this->set_cart_created( $read->cart_created );
	}

	/**
	 * Update a shared cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Shared_Cart|bool Current instance of WCSS_Shared_Cart, false otherwise.
	 */
	public function update() {
		global $wpdb;

		$update = $wpdb->update(
			$wpdb->prefix . $this->table,
			array(
				'cart_key'            => $this->get_cart_key(),
				'cart_value'          => maybe_serialize( $this->get_cart_value() ),
				'cart_shared_by'      => $this->get_cart_shared_by(),
				'cart_retrieved'      => $this->get_cart_retrieved(),
				'cart_retrieved_time' => $this->get_cart_retrieved_time(),
				'cart_created'        => $this->get_cart_created(),
			),
			array(
				'id' => $this->get_id(),
			)
		);

		if ( ! $update ) {
			return false;
		}

		return $this;
	}

	/**
	 * Delete a shared cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Shared_Cart|bool Current instance of WCSS_Shared_Cart, false otherwise.
	 */
	public function delete() {
		global $wpdb;

		$delete = $wpdb->delete(
			$wpdb->prefix . $this->table,
			array(
				'id' => $this->get_id(),
			)
		);

		if ( ! $delete ) {
			return false;
		}

		return $this;
	}
}
