<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

/**
 * Get saved cart object.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Optional. Saved cart key.
 * @return WCSS_Saved_Cart Saved cart object.
 */
function wcss_get_saved_cart( $cart = '' ) {
	return new WCSS_Saved_Cart( $cart );
}

/**
 * Get all saved carts.
 *
 * @since 2.0.0
 *
 * @return array|bool Array of saved cart objects. false otherwise.
 */
function wcss_get_saved_carts() {
	global $wpdb;

	$saved_carts  = array();
	$_saved_carts = $wpdb->get_results(
		"SELECT
			cart_key
		FROM
			{$wpdb->prefix}wcss_saved_cart
		ORDER BY id DESC"
	);

	if ( ! $_saved_carts ) {
		return false;
	}

	foreach ( $_saved_carts as $_saved_cart ) {
		$saved_carts[] = wcss_get_saved_cart( $_saved_cart->cart_key );
	}

	return $saved_carts;
}

/**
 * Get saved carts by fields.
 *
 * @since 2.0.0
 *
 * @param string $field Saved cart field. Like `user_id`, `cart_key`.
 * @param string $value Saved cart field value. Like Saved cart user ID, Saved cart key.
 * @param string $order Optional. ASC or DESC. Default `DESC`.
 * @return array|bool Array of saved cart objects or false if no saved cart found.
 */
function wcss_get_saved_carts_by( $field, $value, $order = 'DESC' ) {
	if ( ! $field || ! $value ) {
		return false;
	}

	if ( 'user_id' !== $field && 'cart_key' !== $field ) {
		return false;
	}

	global $wpdb;

	$saved_carts  = array();
	$_saved_carts = $wpdb->get_results(
		$wpdb->prepare(
			"SELECT cart_key FROM {$wpdb->prefix}wcss_saved_cart WHERE {$field} = %s ORDER BY id {$order}",
			$value
		)
	);

	if ( ! $_saved_carts ) {
		return false;
	}

	foreach ( $_saved_carts as $_saved_cart ) {
		$saved_carts[] = wcss_get_saved_cart( $_saved_cart->cart_key );
	}

	return $saved_carts;
}

/**
 * Delete all saved carts.
 *
 * @since 2.0.0
 *
 * @return void
 */
function wcss_delete_saved_carts() {
	$saved_carts = wcss_get_saved_carts();

	if ( ! $saved_carts ) {
		return;
	}

	foreach ( $saved_carts as $saved_cart ) {
		$shared_cart = wcss_get_shared_cart( $saved_cart->get_cart_key() );
		$shared_cart->delete();
		$saved_cart->delete();
	}
}

/**
 * Delete saved cart.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Saved cart key.
 * @return bool True if saved cart deleted. False otherwise.
 */
function wcss_delete_saved_cart( $cart_key ) {
	$saved_cart  = wcss_get_saved_cart( $cart_key );
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $saved_cart ) {
		return false;
	}

	$saved_cart->delete();
	$shared_cart->delete();

	return true;
}

/**
 * Insert data into saved cart table.
 *
 * @since 2.0.0
 *
 * @param string $args Optional. Args used to create saved cart.
 * @return WCSS_Saved_Cart|bool Saved cart object on success, false otherwise.
 */
function wcss_insert_saved_cart( $args = '' ) {
	$a = wp_parse_args( $args, array(
		'user_id'    => 0,
		'cart_name'  => '',
		'cart_key'   => '',
		'cart_saved' => '',
	) );

	$saved_cart = wcss_get_saved_cart();

	if ( $saved_cart->is_exists() ) {
		return false;
	}

	$saved_cart->set_user_id( $a['user_id'] );
	$saved_cart->set_cart_name( $a['cart_name'] );
	$saved_cart->set_cart_key( $a['cart_key'] );
	$saved_cart->set_cart_saved( $a['cart_saved'] );

	return $saved_cart->create();
}

/**
 * Get saved carts by user ID.
 *
 * @since 2.0.0
 *
 * @param int $user_id User ID.
 * @return array list of saved carts.
 */
function wcss_get_saved_carts_by_user_id( $user_id ) {
	return wcss_get_saved_carts_by( 'user_id', $user_id );
}
