<?php
/**
 * Shared cart meta functions.
 *
 * @package WooCommerce Cart Share & Save
 * @version 1.8.5
 */

defined( 'ABSPATH' ) || exit;

/**
 * Get shared cart meta data.
 *
 * @since 1.8.5
 *
 * @param string $cart_key Shared cart unique key.
 * @param string $meta_key Optional. The meta key to retrieve. By default,
 *                         returns data for all keys. Default empty.
 *
 * @return mixed False for an invalid `$cart_key`.
 *               An empty string if a valid but non-existing Shared cart is passed.
 */
function wcss_get_shared_cart_meta( $cart_key, $meta_key = '' ) {
	global $wpdb;

	if ( ! $cart_key ) {
		return;
	}

	if ( ! $meta_key ) {
		return $wpdb->get_results(
			$wpdb->prepare(
				"SELECT *
				FROM {$wpdb->prefix}wcss_shared_cart_meta
				WHERE cart_key = %s",
				$cart_key
			),
			ARRAY_A
		);
	}

	return $wpdb->get_var(
		$wpdb->prepare(
			"SELECT meta_value
			FROM {$wpdb->prefix}wcss_shared_cart_meta
			WHERE cart_key = %s
			AND meta_key = %s
			ORDER BY id DESC
			LIMIT 1",
			$cart_key,
			$meta_key
		)
	);

}

/**
 * Add shared cart meta data.
 *
 * @since 1.8.5
 *
 * @param string $cart_key   Shared cart unique key.
 * @param string $meta_key   Metadata key.
 * @param mixed  $meta_value Metadata value.
 *
 * @return int|false Meta ID on success, false on failure.
 */
function wcss_add_shared_cart_meta( $cart_key, $meta_key, $meta_value ) {
	global $wpdb;

	if ( wcss_get_shared_cart_meta( $cart_key, $meta_key ) ) {
		return false;
	}

	$query = $wpdb->insert(
		$wpdb->prefix . 'wcss_shared_cart_meta',
		array(
			'cart_key'   => $cart_key,
			'meta_key'   => $meta_key,
			'meta_value' => maybe_serialize( $meta_value ),
		),
		array(
			'%s',
			'%s',
			'%s',
		)
	);

	return $query ? $wpdb->insert_id : false;
}

/**
 * Update shared cart meta data.
 *
 * @since 1.8.5
 *
 * @param string $cart_key   Shared cart unique key.
 * @param string $meta_key   Meta key.
 * @param mixed  $meta_value Meta new value.
 *
 * @return int|bool True on success, false on failure.
 */
function wcss_update_shared_cart_meta( $cart_key, $meta_key, $meta_value ) {
	if ( ! $cart_key || ! $meta_key || ! $meta_value ) {
		return false;
	}

	global $wpdb;

	return $wpdb->update(
		$wpdb->prefix . 'wcss_shared_cart_meta',
		array( 'meta_value' => maybe_serialize( $meta_value ) ),
		array(
			'cart_key' => $cart_key,
			'meta_key' => $meta_key,
		),
		array( '%s' ),
		array(
			'%s',
			'%s',
		)
	);
}

/**
 * Delete shared cart meta.
 *
 * @param string $cart_key Shared cart unique key.
 * @param string $meta_key Optional. Delete meta entry(ies).
 *                         Delete all meta entries if '$meta_key' not passed.
 *
 * @return bool True on success, false on failure.
 */
function wcss_delete_shared_cart_meta( $cart_key, $meta_key = '' ) {
	global $wpdb;

	if ( ! $meta_key ) {
		return $wpdb->delete(
			$wpdb->prefix . 'wcss_shared_cart_meta',
			array( 'cart_key' => $cart_key ),
			array( '%s' )
		);
	}

	return $wpdb->delete(
		$wpdb->prefix . 'wcss_shared_cart_meta',
		array(
			'cart_key' => $cart_key,
			'meta_key' => $meta_key,
		),
		array(
			'%s',
			'%s',
		)
	);
}

/**
 * Delete all shared cart meta.
 *
 * @since 1.8.5
 *
 * @return void
 */
function wcss_delete_all_shared_cart_meta() {
	global $wpdb;

	return $wpdb->query( "TRUNCATE TABLE {$wpdb->prefix}wcss_shared_cart_meta" );
}
